import type { Express } from "express";
import { createServer, type Server } from "http";
import { WebSocketServer, WebSocket } from "ws";
import { storage } from "./storage";
import { signalEngine } from "./signalEngine";
import type { LiveSignal } from "@shared/schema";

export async function registerRoutes(app: Express): Promise<Server> {
  // API Routes
  app.get("/api/signals/recent", async (req, res) => {
    try {
      const signals = await storage.getRecentSignals(50);
      res.json(signals);
    } catch (error) {
      res.status(500).json({ error: "Failed to fetch signals" });
    }
  });

  app.get("/api/market-data", async (req, res) => {
    try {
      const marketData = await storage.getAllMarketData();
      res.json(marketData);
    } catch (error) {
      res.status(500).json({ error: "Failed to fetch market data" });
    }
  });

  app.get("/api/technical-indicators/:pair", async (req, res) => {
    try {
      const { pair } = req.params;
      const indicators = signalEngine.getTechnicalIndicators(decodeURIComponent(pair));
      res.json(indicators);
    } catch (error) {
      res.status(500).json({ error: "Failed to fetch technical indicators" });
    }
  });

  app.get("/api/candle-data/:pair", async (req, res) => {
    try {
      const { pair } = req.params;
      const candles = signalEngine.getCandleData(decodeURIComponent(pair));
      res.json(candles.slice(-50)); // Last 50 candles
    } catch (error) {
      res.status(500).json({ error: "Failed to fetch candle data" });
    }
  });

  app.get("/api/strategy-performance", async (req, res) => {
    try {
      const performance = signalEngine.getStrategyPerformance();
      res.json(performance);
    } catch (error) {
      res.status(500).json({ error: "Failed to fetch strategy performance" });
    }
  });

  const httpServer = createServer(app);

  // WebSocket server for real-time updates
  const wss = new WebSocketServer({ server: httpServer, path: '/ws' });
  
  const clients = new Set<WebSocket>();

  wss.on('connection', (ws) => {
    clients.add(ws);
    console.log('Client connected to WebSocket');
    
    // Send initial data
    ws.send(JSON.stringify({
      type: 'market_status',
      data: { status: 'ACTIVE', timestamp: new Date() }
    }));

    ws.on('close', () => {
      clients.delete(ws);
      console.log('Client disconnected from WebSocket');
    });

    ws.on('error', (error) => {
      console.error('WebSocket error:', error);
      clients.delete(ws);
    });
  });

  // Broadcast function
  function broadcast(message: any) {
    const messageStr = JSON.stringify(message);
    clients.forEach(client => {
      if (client.readyState === WebSocket.OPEN) {
        client.send(messageStr);
      }
    });
  }

  // Generate signals every minute (aligned to minute boundaries)
  function scheduleSignalGeneration() {
    const now = new Date();
    const secondsUntilNextMinute = 60 - now.getSeconds();
    
    setTimeout(() => {
      generateAndBroadcastSignals();
      
      // Then repeat every minute
      setInterval(generateAndBroadcastSignals, 60000);
    }, secondsUntilNextMinute * 1000);
  }

  async function generateAndBroadcastSignals() {
    try {
      const pairs = signalEngine.getAllPairs();
      const newSignals: LiveSignal[] = [];
      
      // Add new candles and generate signals
      for (const pair of pairs) {
        signalEngine.addNewCandle(pair);
        
        // Generate signal with some probability
        if (Math.random() < 0.4) { // 40% chance per pair per minute for more signals
          const signal = signalEngine.generateSignal(pair);
          if (signal && signal.confidence >= 75) { // Lower threshold for more signals
            newSignals.push(signal);
            
            // Store signal in database
            const storedSignal = await storage.createSignal({
              pair: signal.pair,
              direction: signal.direction,
              confidence: signal.confidence,
              strategies: signal.strategies,
              martingaleStep: 1,
              result: null,
              pnl: null,
            });
            
            // Auto-resolve signals after 1 minute with simulated results
            setTimeout(async () => {
              const isWin = Math.random() < (signal.confidence / 100); // Use confidence as win probability
              const result = isWin ? 'WIN' : 'LOSS';
              const pnl = isWin ? 8.5 : -10; // 85% payout or loss
              
              await storage.updateSignalResult(storedSignal.id, result, pnl);
              
              // Broadcast updated signal results
              broadcast({
                type: 'signal_result_update',
                data: { id: storedSignal.id, result, pnl }
              });
            }, 60000); // 1 minute
          }
        }
      }
      
      // Broadcast new signals
      if (newSignals.length > 0) {
        broadcast({
          type: 'new_signals',
          data: newSignals
        });
      }
      
      // Broadcast updated market data
      const marketData = await storage.getAllMarketData();
      broadcast({
        type: 'market_data_update',
        data: marketData
      });
      
      // Broadcast time update
      broadcast({
        type: 'time_update',
        data: { timestamp: new Date() }
      });
      
    } catch (error) {
      console.error('Error generating signals:', error);
    }
  }

  // Update prices every 2 seconds
  setInterval(async () => {
    try {
      const pairs = signalEngine.getAllPairs();
      const updates = [];
      
      for (const pair of pairs) {
        const candles = signalEngine.getCandleData(pair);
        const lastCandle = candles[candles.length - 1];
        
        if (lastCandle) {
          // Simulate small price movements within the current candle
          const volatility = pair.includes('BTC') ? 50 : pair.includes('JPY') ? 0.1 : 0.0002;
          const priceChange = (Math.random() - 0.5) * volatility;
          const newPrice = lastCandle.close + priceChange;
          
          updates.push({ pair, price: newPrice });
        }
      }
      
      if (updates.length > 0) {
        broadcast({
          type: 'price_updates',
          data: updates
        });
      }
    } catch (error) {
      console.error('Error updating prices:', error);
    }
  }, 2000);

  // Start signal generation immediately and then every minute
  generateAndBroadcastSignals(); // Generate signals immediately on startup
  scheduleSignalGeneration();

  return httpServer;
}
